#include <stdio.h>
#include <stdlib.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "Maths.h"
#include "Vec2f.h"

#define SCREEN_WIDTH    640
#define SCREEN_HEIGHT   480

void drawShapeLeft(Vec2f* points, int nbPoints, Color color)
{
    for (int i = 0; i < nbPoints; i++)
    {
        int next = (i + 1) % nbPoints;
        gfx.line(points[i].x + SCREEN_WIDTH  / 3,
                 points[i].y + SCREEN_HEIGHT / 2,
                 points[next].x + SCREEN_WIDTH  / 3,
                 points[next].y + SCREEN_HEIGHT / 2,
                 color);
    }
}

void drawShapeRight(Vec2f* points, int nbPoints, Color color)
{
    for (int i = 0; i < nbPoints; i++)
    {
        int next = (i + 1) % nbPoints;
        gfx.line(points[i].x + SCREEN_WIDTH  * 2 / 3,
                 points[i].y + SCREEN_HEIGHT / 2,
                 points[next].x + SCREEN_WIDTH  * 2 / 3,
                 points[next].y + SCREEN_HEIGHT / 2,
                 color);
    }
}

void translateShape(Vec2f* points, int nbPoints, Vec2f transVector)
{
    for (int i = 0; i < nbPoints; i++)
        points[i] += transVector;
}

void scaleShape(Vec2f* points, int nbPoints, Vec2f scaleVector)
{
    for (int i = 0; i < nbPoints; i++)
        points[i].scale(scaleVector);
}

void rotateShape(Vec2f* points, int nbPoints, float angle)
{
    for (int i = 0; i < nbPoints; i++)
        points[i].rotate(angle);
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("2D Transforms 9", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();

    // define the shape
    Vec2f shape[] =
    {
        {-20, 20}, {-20, -20}, {30, 20}
    };

    float angle = 0;

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

        drawShapeLeft(shape, 3, Color(0, 255, 255));
        drawShapeRight(shape, 3, Color(0, 255, 255));

        // copy shape
        Vec2f shape2[3];

        for (int i = 0; i < 3; i++)
            shape2[i] = shape[i];

        // transform and draw
        rotateShape(shape2, 3, DEG_TO_RAD(angle));
        drawShapeLeft(shape2, 3, Color(0, 255, 0));

        // copy shape
        Vec2f shape3[3];

        for (int i = 0; i < 3; i++)
            shape3[i] = shape[i];

        // transform and draw
        translateShape(shape3, 3, -shape[0]);
        rotateShape(shape3, 3, DEG_TO_RAD(angle));
        translateShape(shape3, 3, shape[0]);
        drawShapeRight(shape3, 3, Color(255, 0, 0));

        angle += 1;

        gfx.render();
        sys.wait(20);
        sys.processEvents();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
